#warn "[DEPRECATION] Raykit::VersionHelper is deprecated. Use bump gem for better transparency."

module Raykit
    # Version functionality
    class Version
        attr_accessor :start_time

        # detect a version number based on the NAME variable, if defined
        def self.detect(name,verbose=false)
            version=detect_from_file("#{name}/#{name}.csproj",/<Version>([-\w\d\.]+)</,verbose)
            return version if(version.length>0)

            version=detect_from_file("#{name}/Properties/AssemblyInfo.cs",/^\[assembly: AssemblyVersion\(\"([.\w]+)/,verbose)
            return version if(version.length>0)

            version=detect_from_file("#{name}.gemspec",/version[\s]+=[\s]+['"]([\w.]+)['"]/,verbose)
            return version if(version.length>0)

            version=detect_from_file("#{name}.gemspec",/version\s?=\s?['|"]([.\d]+)['|"]/,verbose)
            return version if(version.length>0)

            version=detect_from_file("#{name}.nuspec",/<[Vv]ersion>([\d.]+)</,verbose)
            return version if(version.length>0)

            if(verbose)
                warning="\u26A0"
                symbol=Rainbow(warning.encode('utf-8')).yellow
                puts symbol + ' version not detected'
            end
            ''
        end

        def self.detect_from_file(filename,regex,verbose)
            version=''
            if(File.exists?(filename))
                match = IO.read(filename).match(regex)
                if(!match.nil? && match.captures.length>0)
                    version = match.captures[0]
                end
            else
                return ''
            end
            if(verbose)
                if(version.length > 0)
                    puts "version detected in #{filename}"
                else
                    puts "no version detected in #{filename}, regex #{regex.source}"
                end
            end
            version
        end

        def self.set_version_in_file(filename,version)
            text=IO.read(filename)
            if(filename.include?('.gemspec'))
                new_text=text.gsub(/version\s?=\s?['|"]([.\d]+)['|"]/,"version='#{version}'")
            end
            if(filename.include?('.csproj'))
                new_text=text.gsub(/<Version>([-\w\d.]+)</,"<Version>#{version}<")
            end
            #new_text=text.gsub(/<Version>([-\w\d.]+)</,"<Version>#{version}<")
            #new_text=new_text.gsub(/version[\s]+=\s?['|"]([.\d]+)['|"]/,"version='#{version}'")
            #new_text=new_text.gsub(/Version="([.\d]+)"/,"Version=\"#{version}\"")
            File.open(filename,'w'){|f|f.write(new_text)} if(new_text!=text)
        end

        def self.sync_file_versions(source_filename,destination_filename)
            version = Version.detect_from_file(source_filename,/<Version>([-\w\d.]+)</,false)
            Version.set_version_in_file(destination_filename,version)
        end

        # increment to last digit of a version string
        def self.bump(version)
            # major.minor[.build[.revision]]  (example: 1.2.12.102)
            version_ints = version.split('.').map{|s| s.to_i}
            version_ints[-1] = version_ints.last + 1;
            version_ints.map{|i| i.to_s}.join('.')
        end

        def self.bump_file(filename)
            if(!File.exist?(filename))
                warn "Raykit::Version.bump_file filename '#{filename}' does not exist."
            end

            old_version=''
            if(filename.include?('.gemspec'))
                old_version=detect_from_file(filename,/version\s?=\s?['|"]([.\d]+)['|"]/,false)
            end
            if(filename.include?('.csproj'))
                old_version=detect_from_file(filename,/<Version>([-\w\d.]+)</,false)
            end

            if(old_version.length > 0)
                new_version = bump(old_version)
                set_version_in_file(filename,new_version)
            end

            new_version
        end
    end
end